// Copyright (c).  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

library tracker.models;

import 'package:polymer/polymer.dart';

final appModel = new Tracker();

/**
 * A model for the tracker app.
 *
 * [ukoly] contains all tasks used in this app.
 */
class Tracker extends Observable {
  @observable List<Ukol> ukoly;
}

/**
 * A model for creating a single task.
 *
 * A task can be saved or unsaved. Only a saved task has a taskID.
 *
 * This model defines validation rules for a Task. It is the responsibility of
 * the view layer to validate a task before assigning a taskID to the task. A
 * task with a taskID is considered saved.
 */
class Ukol extends Observable {
  static bool POVINNY_NAZEV = true;
  static const MAX_NAZEV_DELKA = 40;
  static const MAX_POPIS_DELKA = 200;
  static const SOUCASNE = 'Současné';
  static const NEVYRIZENE = 'Nevyřízené';
  static const HOTOVE = 'Hotové';

  @observable int ukolID;
  @observable String nazev = '';
  @observable String popis = '';
  @observable String status = NEVYRIZENE;
  @observable DateTime vytvoreno;
  @observable DateTime update;

  Ukol.unsaved();

  Ukol(this.nazev, this.popis, this.status);

  bool get isValid {
    var minTitleLength = POVINNY_NAZEV ? 1 : 0;
    return (nazev.length >= minTitleLength &&
        nazev.length < MAX_NAZEV_DELKA &&
        popis.length < MAX_POPIS_DELKA);
  }

  bool get ulozeno => ukolID != null;
}
