// Copyright (c) 2013, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

library tracker.web.task_form_element;

import 'package:polymer/polymer.dart';
import 'package:tracker/models.dart';
import 'dart:html';
import 'dart:math';

@CustomTag('task-form-element')
class TaskFormElement extends PolymerElement {
  bool get applyAuthorStyles => true;
  @observable Ukol ukol;
  @observable String titleErrorMessage = '';
  @observable int maxTitleLength = Ukol.MAX_NAZEV_DELKA;
  @observable String descriptionErrorMessage = '';
  @observable int maxDescriptionLength = Ukol.MAX_POPIS_DELKA;
  @observable final List<String> taskStatusOptions = toObservable([
      Ukol.SOUCASNE, Ukol.NEVYRIZENE, Ukol.HOTOVE]);
  @observable int statusSelectedIndex = 1;
  @observable String previousStatus = '';
  @observable String submitPopisek = '';

  TaskFormElement.created() : super.created();

  attached() {
    super.attached();
    submitPopisek = ukol.ulozeno ? 'Aktualizovat' : 'Vytvořit';
    statusSelectedIndex = taskStatusOptions.indexOf(ukol.status);
    if (!ukol.ulozeno) {
      statusSelectedIndex = taskStatusOptions.indexOf(ukol.status);
      previousStatus = ukol.status;
    }
  }

  bool validateTitle() {
    int len = ukol.nazev.length;
    bool valid = false;
    if (len == 0 && Ukol.POVINNY_NAZEV) {
      titleErrorMessage = 'Název  je povinný';
    } else if (len > maxTitleLength) {
      titleErrorMessage = 'Název musí být méně než $maxTitleLength znaků';
    } else {
      titleErrorMessage = '';
      valid = true;
    }
    return valid;
  }

  bool validateDescription() {
    int len = ukol.popis.length;
    bool valid = false;
    if (len >= maxDescriptionLength) {
      descriptionErrorMessage = 'Popis musí být méně než '
          '$maxDescriptionLength znaků';
    } else {
      descriptionErrorMessage = '';
      valid = true;
    }
    return valid;
  }

  createOrUpdateTask(Event event, detail, sender) {
    event.preventDefault();
    event.stopPropagation();

    // Do view validation to trigger error messages, and model validation
    // to ensure that a task is never in an invalid state.
    if (!validateTitle() || !validateDescription() || !ukol.isValid) return;

    previousStatus = ukol.status;
    ukol.status = taskStatusOptions[statusSelectedIndex];

    if (ukol.ulozeno) {
      updateTask();
    } else {
      createTask();
    }
    dispatchNotNeeded();
  }

  void dispatchNotNeeded() {
    dispatchEvent(new CustomEvent('notneeded'));
  }

  void dispatchCancelForm() {
    dispatchEvent(new CustomEvent('cancelform'));
  }

  createTask() {
    DateTime now = new DateTime.now();
    var random = new Random();
    ukol.ukolID = random.nextInt(1000 * 1000);
    ukol.vytvoreno = now;
    ukol.update = now;
    appModel.ukoly.add(ukol);
  }

  updateTask() {
    DateTime now = new DateTime.now();
    ukol.update = now;
    if (previousStatus != ukol.status) {
      appModel.ukoly.remove(ukol);
      appModel.ukoly.add(ukol);
    }
  }

  deleteTask() {
    if (window.confirm('Jsi si jist smazáním?')) {
      appModel.ukoly.remove(ukol);
    }
  }
}
