// Copyright (c) 2013, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

library tracker.web.tracker_app;

import 'package:polymer/polymer.dart';
import 'package:tracker/models.dart';
import 'package:tracker/seed.dart' as seed;

@CustomTag('tracker-app')
class TrackerApp extends PolymerElement {
  bool get applyAuthorStyles => true;
  @observable final ObservableList<Ukol> ukoly = toObservable([]);
  @observable Tracker app;
  @observable Ukol novyUkol = new Ukol.unsaved();
  @observable String hledatParam = '';
  @observable bool pouzForm = false;

  // Buckets for grouping tasks by status.
  @observable List<Ukol> soucasne = toObservable([]);
  @observable List<Ukol> nevyrizene = toObservable([]);
  @observable List<Ukol> hotove = toObservable([]);

  // Used in search for stashing elements from tasks that don't match the
  // search param.
  List<Ukol> filteredOutTasks = [];

  TrackerApp.created() : super.created() {
    appModel.ukoly = ukoly;

    ukoly.changes.listen((List<ChangeRecord> changes) {
      _filterTasksByStatus();
    });

    _addSeedData();
  }

  toggleFormDisplay() {
    pouzForm = !pouzForm;
  }

  showEmptyForm() {
    novyUkol = new Ukol.unsaved();
    if (!pouzForm) {
      toggleFormDisplay();
    }
  }

  search() {
    ukoly.addAll(filteredOutTasks);
    filteredOutTasks.clear();

    for (Ukol ukol in ukoly) {
      if (!taskMatchesSearchParam(ukol)) {
        filteredOutTasks.add(ukol);
      }
    }

    for (Ukol ukol in filteredOutTasks) {
      ukoly.remove(ukol);
    }
  }

  bool taskMatchesSearchParam(Ukol ukol) {
    var param = hledatParam.toLowerCase();
    if (param.isEmpty) return true;
    return ukol.nazev.toLowerCase().contains(param) ||
        ukol.popis.toLowerCase().contains(param);
  }

  _addSeedData() {
    for (var i = 0; i < seed.data.length; i++) {
      // Assign IDs to the seed data, so that task.saved == true.
      seed.data[i].ukolID = i;
      ukoly.add(seed.data[i]);
    }
  }

  _filterTasksByStatus() {
    soucasne = [];
    nevyrizene = [];
    hotove = [];
    for (Ukol ukol in ukoly) {
      if (ukol.status == Ukol.SOUCASNE) {
        soucasne.add(ukol);
      } else if (ukol.status == Ukol.NEVYRIZENE) {
        nevyrizene.add(ukol);
      } else {
        hotove.add(ukol);
      }
    }
  }
}